@@ -26,7 +26,7 @@ Lingua::EN::AddressParse - manipulate geographical addresses
 
     print $my_address{suburb};                  # ST JOHNS WOOD
 
-    %my_formatted_address = $address->case_components;
+    %my_formatted_address = $address->components;
     print $my_formatted_address{street_name};        # Old South Head
     print $my_formatted_address{street_type};   # St
 
@@ -213,8 +213,12 @@ This method is a prerequisite for all the following methods.
 
 =head2 components
 
-    %address = $address->components;
+    %address = $address->components($upper_case_all);
     $suburb = $address{suburb};
+    
+If the optional argument $upper_case_all is set to a postive value, all components
+are converted to upper case.
+    
 
 The C<components> method returns all the address components in a hash. The
 following keys are used for each component:
@@ -228,6 +232,8 @@ following keys are used for each component:
     sub_property_identifier
     property_identifier
     property_name
+    level - such as 12th Floor
+    building - such as Tower A
     street_direction_prefix (such as East, NW, North etc)
     base_street_name (the name with direction removed, such as "Main" in "East Main St")
     street_name (the full street name such as "East Main")
@@ -241,13 +247,7 @@ following keys are used for each component:
 If a component has no matching data for a given address, it's values will be
 set to the empty string.
 
-=head2 case_components
-
-   %my_address = $address->case_components;
-   $cased_suburb = $my_address{suburb};
-
-The C<case_components> method does the same thing as the C<components> method,
-but each component is converted to title case, meaning the first letter of each
+Each component is converted to title case, meaning the first letter of each
 component is set to capitals and the remainder to lower case.
 
 Proper name capitalisations such as MacNay and O'Brien are observed
@@ -260,12 +260,15 @@ The following components are not converted to title case:
     post_code
     country
     street_direction_suffix
+    
+If your input data is all upper case and you want to retian that format for parsed
+data, you will need to apply the 'uc' function to each component.
 
 =head2 case_all
 
     $correct_casing = $address->case_all;
 
-The C<case_all> method does the same thing as the C<case_components> method except
+The C<case_all> method does the same thing as the C<components> method except
 the entire address is returned as a title cased text string.
 
 If the force_case option was set in the C<new> method above, address case the
@@ -408,7 +411,7 @@ use Lingua::EN::AddressParse::Grammar;
 use Lingua::EN::NameParse;
 use Parse::RecDescent;
 
-our $VERSION = '1.23';
+our $VERSION = '1.24';
 
 #------------------------------------------------------------------------------
 # Create a new instance of an address parsing object. This step is time
@@ -463,21 +466,28 @@ sub parse
 
     my $pre_cursor;
     ($pre_cursor,$address->{input_string}) = _extract_precursor($address->{input_string});
-
-    my $po_box_type;
-    ($po_box_type,$address->{input_string}) = _extract_po_box_type($address->{input_string});
-
-
+    
     # Replace commas (which can be used to chunk sections of addresses) with spaces
     $address->{input_string} =~ s/,/ /g;
-
+    
     if ( $address->{auto_clean} )
     {
         $address->{input_string} = _clean($address);
-    }
+    }    
+
+    my $po_box_type;
+    ($po_box_type,$address->{input_string}) = _extract_po_box_type($address->{input_string});
     
-    # Normalise sub property ID, 4/22-24 => UNIT 4 22-24
-    if ($address->{country_code} ne 'US' and $address->{input_string} =~ /^(\d{1,4}[A-Z]{0,2})[\/| ](\d.*)$/ )
+    my $level;
+    ($level,$address->{input_string}) = _extract_level($address->{input_string});
+    
+    my $building;
+    ($building,$address->{input_string}) = _extract_building($address->{input_string});    
+    
+
+    
+    # Normalise sub property ID, 4/22-24 => UNIT 4 22-24, 4 12 => Unit 4 12
+    if ($address->{country_code} ne 'US' and $address->{input_string} =~ /^(\d{1,4}[A-Z]{0,2})[\/| ](\d+[ \w-].*)$/ )
     {
         $address->{input_string} = "UNIT $1 $2";     
     }    
@@ -488,35 +498,37 @@ sub parse
     # This space will be removed in the _assemble function
     $address->{input_string} .= ' ';
 
-    $address = _assemble($address,$pre_cursor,$po_box_type);
+    $address = _assemble($address,$pre_cursor,$po_box_type,$level,$building);
     _validate($address);
 
 
     return($address,$address->{error});
 }
-#------------------------------------------------------------------------------
-# Return all address components as a hash
-sub components
-{
-    my $address = shift;
-    return(%{ $address->{components} });
-}
+
 #------------------------------------------------------------------------------
 # Apply correct capitalisation to each component of an address
 
-sub case_components
+sub components
 {
     my $address = shift;
+    my ($uc_all) = @_;
 
-    my %orig_components = $address->components;
+    my %orig_components = %{ $address->{components} };
 
     my (%cased_components);
     foreach my $curr_key ( keys %orig_components )
     {
         my $cased_value;
         my $curr_value = $orig_components{$curr_key};
+        
+        if ($uc_all)
+        {
+             $cased_components{$curr_key} = uc($curr_value);
+            next;            
+        }
+        
 
-        if ( $curr_key =~ /^(base_street_name|street_name|street_type|suburb|property_name|sub_property|pre_cursor|po_box_type)/ )
+        if ( $curr_key =~ /^(base_street_name|street_name|street_type|suburb|property_name|sub_property|pre_cursor|po_box_type|level|building)/ )
         {
 
             if ( $curr_key eq 'street_name' and$curr_value =~ /^US HIGHWAY (.*)/ )
@@ -566,7 +578,7 @@ sub case_components
                 }
             }
         }
-       # retain street_direction,sub country and countries capitalisation, usually uppercase
+        # retain street_direction,sub country and countries capitalisation, usually uppercase
         elsif ($curr_key =~ /street_direction/)
         {
             if (length($curr_value) == 1 or length($curr_value) == 2)
@@ -616,14 +628,14 @@ sub case_all
         );
         if ( $address->{country} eq 'US' )
         {
-           $component_order{'suburban'} = [ qw/pre_cursor property_identifier street_name  street_type street_direction_suffix sub_property_type sub_property_identifier suburb subcountry post_code country/];
+           $component_order{'suburban'} = [ qw/pre_cursor property_identifier street_name  street_type street_direction_suffix  building level sub_property_type sub_property_identifier suburb subcountry post_code country/];
         }
         else
         {
-           $component_order{'suburban'} = [ qw/pre_cursor sub_property_type sub_property_identifier property_identifier street_name  street_type suburb subcountry post_code country/ ];
+           $component_order{'suburban'} = [ qw/pre_cursor building level sub_property_type sub_property_identifier property_identifier street_name  street_type suburb subcountry post_code country/ ];
         }
 
-        my %component_vals = $address->case_components;
+        my %component_vals = $address->components;
         my @order = @{ $component_order{$address->{properties}{type} } };
 
         foreach my $component ( @order )
@@ -687,7 +699,7 @@ sub report
 
 
     _fmt_report_line(\$report,"COMPONENTS",'');
-    my %comps = $address->case_components;
+    my %comps = $address->components;
     foreach my $comp ( sort keys %comps)
     {
         if (defined($comps{$comp})  )
@@ -710,7 +722,7 @@ sub _assemble
 {
 
     my $address = shift;
-    my ($pre_cursor,$po_box_type) = @_;
+    my ($pre_cursor,$po_box_type,$level,$building) = @_;
 
     # Parse the address according to the rules defined in the AddressParse::Grammar module,
     # $::RD_TRACE  = 1;  # for debugging RecDescent output
@@ -728,6 +740,25 @@ sub _assemble
     {
         $address->{components}{'pre_cursor'} = '';
     }
+    
+    if ($level)
+    {
+        $address->{components}{'level'} = $level;
+    }
+    else
+    {
+        $address->{components}{'level'} = '';
+    }
+    
+    if ($building)
+    {
+        $address->{components}{'building'} = $building;
+    }
+    else
+    {
+        $address->{components}{'building'} = '';
+    }       
+        
 
     if ($po_box_type)
     {
@@ -737,7 +768,6 @@ sub _assemble
     {
         $address->{components}{'po_box_type'} = '';
     }
-
   
 
     $address->{components}{post_box} = '';
@@ -763,12 +793,18 @@ sub _assemble
     
     if ( $parsed_address->{sub_property} )
     {
-        if ($parsed_address->{sub_property} =~ /^(#|\w{1,}) (.*)$/ )
+        if ($parsed_address->{sub_property} =~ /^(#|[A-Z]{1,}) (.*)$/ )
         {
             # Such as Unit 24, # 4A etc
             $address->{components}{sub_property_type} = $1;
             $address->{components}{sub_property_identifier} = $2;
         }
+        elsif ($parsed_address->{sub_property} =~ /^(\d\w\w) (.*)$/ )
+        {
+            # Such as 1st Floor
+            $address->{components}{sub_property_type} = $2;
+            $address->{components}{sub_property_identifier} = $1;
+        }        
     }
 
     $address->{components}{property_identifier} = '';
@@ -974,8 +1010,11 @@ sub _clean
 
     # Remove annotations enclosed in brackets, such as 1 Smith St (Cnr Brown St)
     $input =~ s|\(.*\)||;
+   
+   # Normalise half house numbers, sucvh as 12.5 to 12 1/2. This is needed now before full stops are stripped out         
+    $input =~ s|^(\d{1,4})\.5 |$1 1/2 |;        
 
-    # remove illegal characters
+    # strip full stops, remove illegal characters
     # & can be part of property name
     # hash (#) may denote number for USA address
     # quotes can occur as property name delimiters
@@ -986,16 +1025,11 @@ sub _clean
     $input =~ s|  +| |go ;
     $input =~ s|^ ||;
     $input =~ s| $||;
-    
-    # remove redundant padding around sub proprty and property IDs
-    $input =~ s| -|-|;
-    $input =~ s|- |-|;
-    $input =~ s| /|/|;
-    $input =~ s|/ |/|;
+
 
     # Expand abbreviations that are too short
 
-    # street nouns
+    $input =~ s/LAKE ST (GEORGE|CLAIR)/LAKE SAINT $1/; # otherwise St gets consumed to early as 'Street'
     $input =~ s| CSEWY | CAUSEWAY |;
 
     # street types
@@ -1008,12 +1042,11 @@ sub _clean
     $input =~ s|^UN? |UNIT |;
     $input =~ s|^U(\d+)|UNIT $1|;
 
-
     # Fix badly formed number dividers such as home unit format of 14/ 12 becomes 14/12, 2- 7A becomes 2-7A
     $input =~ s|/ |/|;
     $input =~ s| /|/|;
     $input =~ s|- |-|;
-    $input =~ s| -|/|;
+    $input =~ s| -|-|;    
 
     # Remove redundant spaces in property identifiers,  21 B Smith St becomes 21B Smith St
 
@@ -1035,7 +1068,7 @@ sub _clean
     # Add or remove spaces in sub property identifiers
     if ( $address->{country_code} eq 'US' )
     {
-        # Fix US sub property identifiers that appear after stree name and type
+        # Fix US sub property identifiers that appear after street name and type
         # add space between # and the number so #2 becomes '# 2'
         $input =~ s| #(\d)| # $1|;
         $input =~ s| #([A-Z])| # $1|;
@@ -1064,13 +1097,14 @@ sub _clean
     # remove dash that is not from a sequence, such as D-5 or 22-A
     $input =~ s|([A-Z])-(\d)|$1$2|;
     $input =~ s|(\d)-([A-Z])|$1$2|;
+    
 
     return($input);
 }
 #-------------------------------------------------------------------------------
 # Remove any "care of" type of precursor from the main address
 # such as: C/O BRAKEFIELD  BETTY S PO BOX 214 GULF HAMMOCK, FL 32639-0214
-# It will be saved as an address attribute
+# It will later be saved as an attribute in the address object
 
 sub _extract_precursor
 {
@@ -1089,6 +1123,62 @@ sub _extract_precursor
         return('',$input)
     }
 }
+#-------------------------------------------------------------------------------
+# Remove any level or floor info such as:
+# 12 Smith St Floor 2
+# Level 22 Suite 3 12 Main St
+# It will later be saved as an attribute in the address object
+
+sub _extract_level
+{
+    my ($input) = @_;
+    my ($level);
+    
+    if    
+    (
+        # Level info could be at start of string so first space is optional
+        $input =~ / ?(\d{1,2}(ST|ND|RD|TH) (FLOOR|FLR|FL) )/ or       
+        $input =~ / ?(LEVEL (\d{1,2}|[GM])[\/ -])/ or
+        $input =~ / ?((FLOOR|FLR|FL) \d{1,2}[\/ -])/
+     )
+    {
+        $level = $1;
+        $level =~ s|/||;
+        $level =~ s|-||;
+        $input =~ s/$level//;
+    }
+ 
+    return($level,$input);
+}
+#-------------------------------------------------------------------------------
+# Remove any building info such as:
+# Building 2 Level 12 123 Smith St
+# 12 Main St Tower A Level 2
+# It will later be saved as an attribute in the address object
+
+sub _extract_building
+{
+    my ($input) = @_;
+    my ($building);
+    
+    my $bld = qr{BLOCK|BLDG?|BUILDING|TOWER};
+    
+    if    
+    (
+        $input =~ / ?(($bld) ([A-Z]{1,2}|\d+) )/ or  # BLD 12 or AA
+        $input =~ / ?(($bld) \d{1,3}[A-Z] )/ or      # BLD 32C
+        $input =~ / ?(($bld) [A-Z]\d{1,3} )/         # BLD C12       
+     )
+    {
+        $building = $1;
+        $building =~ s|/||;
+        $building =~ s|-||;
+        $input =~ s/$building//;
+    }
+ 
+    return($building,$input);
+}
+
 #-------------------------------------------------------------------------------
 # Remove any description that follows the suburb after the main address
 # such as: PO BOX 1305 BIBRA LAKE PRIVATE BOXES WA 6965"
@@ -1099,7 +1189,7 @@ sub _extract_po_box_type
     my ($input) = @_;
     my ($po_box_type,$address_start,$address_end);
 
-    if ($input =~ /^(.*? )(PRIVATE BOXES)( .*)$/ )
+    if ($input =~ /^(.*?) (PRIVATE BOXES)( .*)$/ )
     {
         $address_start = $1;
         $po_box_type = $2;
@@ -1124,7 +1214,7 @@ sub _trim_trailing_space
     {
         if ($address->{components}{$key} )
         {
-            $address->{components}{$key}  =~ s/ $//;
+            $address->{components}{$key}  =~ s/ $//g;
         }
     }         
 }
@@ -1145,7 +1235,8 @@ sub _check_vowel
     my @words = split(/ /,$str);
     foreach my $word (@words)
     {
-        if ( length($word) > 1 and $word !~ /[AEIOUY]|ST|MT/ )  # St is abbreviation for Saint
+        #  Saint, Mount, Junior, Senior (as in Martin Luther KIng Snr)
+        if ( length($word) > 1 and $word !~ /[AEIOUY]|ST|MT|JN?R|SN?R/ )  
         {
             return(1);
         }
