@@ -35,7 +35,7 @@ use strict;
 use warnings;
 use Locale::SubCountry;
 
-our $VERSION = '1.23';
+our $VERSION = '1.24';
 
 #-------------------------------------------------------------------------------
 # Rules that define valid orderings of an addresses components
@@ -269,30 +269,21 @@ q{
 my $sub_property =
 q{
 
-    # Level 4 Tower A,
-    # Unit 3A Block C
     sub_property:
 
-        /LEVEL (\d+|[GM]) (BUILDING|TOWER) \d+[A-Z]? / |
-        /LEVEL (\d+|[GM]) (BUILDING|TOWER) [A-Z] / |
-        /UNIT \d+[A-Z]? (BLOCK|BLDG|BUILDING|TOWER) ([A-Z]|\d+) / |
-        /\d{1,2}(ST|ND|RD|TH) (FLOOR|FLR|FL) / |
-        /(FLOOR|FLR|FL) \d{1,2} /
+        /SUITE \w+ /      
         |
         sub_property_type unit_number
         {
            $return = "$item[1]$item[2]"
-        }
- 
+        } 
 
-
-    # Unit 34, BLDG 12, SUITE A,
+    # Unit 34, Shop 12C
 
     sub_property_type:
         /(
         APARTMENT   | APT   |
         BAY         |
-        BUILDING    | BLDG  |
         DEPARTMENT  |
         FACTORY     |
         FLAT        |
@@ -307,7 +298,6 @@ q{
         LOT         |
         OFFICE      |
         OFC         |
-        LEVEL       |
         LOT         |
         NO          |
         PENTHOUSE   |
@@ -331,7 +321,7 @@ q{
         /\d{1,4}[A-Z]{0,2} / | # such as 23B, 6AW
         /\d{1,2}[A-Z]\d /    | # such as 4A5
         /[A-Z]{1,2}\d{0,4} / | # such as # D512
-        /\d{1,3}-\d{1,3}/        
+        /\d{1,3}-\d{1,3}/      # such as # 200-204
 };
 
 #------------------------------------------------------------------------------
@@ -401,21 +391,22 @@ q{
 my $street =
 q{
 
-    # Streets with no street type such as Road, Lane etc. Eg: Broadway, Highway 19, Avenue C
+    # Streets with no street type such as Road, Lane etc.  
     street_untyped :
 
         major_road |
-        avenue_letter |
+        avenue_ordinal |
         street_name_single_word |
         street_noun |
         french_style |
-        /AVENUE OF \w+ (\w+ )?/  # The Americas, Two Roivers wtc
+        /AVENUE OF \w+ \w+ /   # The Americas, Two Rivers etc       
         
     major_road :
         /([N|E|S|W] )?(COUNTY |STATE |US |FIRE )?(ALT|HIGHWAY|LANE|HWY|ROAD|RD|ROUTE) \d{1,3}\w? ([N|E|S|W|NORTH|EAST|SOUTH|WEST] )?/
 
-    avenue_letter :
-        /([N|E|S|W] )?AVENUE [A-Z] /
+    # Avenue C, 12 1/2 etc
+    avenue_ordinal :
+        /([N|E|S|W] )?AVENUE ([A-Z]|\d{1,2}( 1\/2)?) /
 
     street_name_single_word:
         /([N|E|S|W] )?BROADWAY|BOARDWALK|BOULEVARD|BOWERY|ESPLANADE|KINGSWAY|QUEENSWAY|GREENWAY|PARKWAY|RIVERBANK /
@@ -425,7 +416,7 @@ q{
         }
         
     street_noun:
-        /THE / any_word
+        /(THE|VIA) / any_word
         {
             $return = "$item[1]$item[2]"
         }
@@ -490,6 +481,7 @@ q{
         CAY|
         CHASE|
         CIRCLE|
+        CENTRAL|
         CLUB|
         CREST|
         CRESCENT|
@@ -697,6 +689,7 @@ q{
         LOOP|
         MEWS|
         OVERFLOW|
+        OVERLOOK|
         OVAL|
         PASS|
         PASSAGE|PSGE|PSG|
